package sun.nio.fs;

import java.nio.file.attribute.GroupPrincipal;
import java.nio.file.attribute.UserPrincipalNotFoundException;
import java.io.IOException;
import java.nio.file.attribute.UserPrincipal;

class WindowsUserPrincipals
{
    private WindowsUserPrincipals() {
    }
    
    static UserPrincipal fromSid(long n) throws IOException {
        String convertSidToStringSid;
        try {
            convertSidToStringSid = WindowsNativeDispatcher.ConvertSidToStringSid(n);
            if (convertSidToStringSid == null) {
                throw new AssertionError();
            }
        }
        catch (WindowsException ex) {
            throw new IOException("Unable to convert SID to String: " + ex.errorString());
        }
        WindowsNativeDispatcher.Account lookupAccountSid = null;
        String string;
        try {
            lookupAccountSid = WindowsNativeDispatcher.LookupAccountSid(n);
            string = lookupAccountSid.domain() + "\\" + lookupAccountSid.name();
        }
        catch (WindowsException ex2) {
            string = convertSidToStringSid;
        }
        int n2 = (lookupAccountSid == null) ? 8 : lookupAccountSid.use();
        if (n2 == 2 || n2 == 5 || n2 == 4) {
            return new Group(convertSidToStringSid, n2, string);
        }
        return new User(convertSidToStringSid, n2, string);
    }
    
    static UserPrincipal lookup(String str) throws IOException {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("lookupUserInformation"));
        }
        int lookupAccountName;
        try {
            lookupAccountName = WindowsNativeDispatcher.LookupAccountName(str, 0L, 0);
        }
        catch (WindowsException ex) {
            if (ex.lastError() == 1332) {
                throw new UserPrincipalNotFoundException(str);
            }
            throw new IOException(str + ": " + ex.errorString());
        }
        assert lookupAccountName > 0;
        NativeBuffer nativeBuffer = NativeBuffers.getNativeBuffer(lookupAccountName);
        try {
            if (WindowsNativeDispatcher.LookupAccountName(str, nativeBuffer.address(), lookupAccountName) != lookupAccountName) {
                throw new AssertionError("SID change during lookup");
            }
            return fromSid(nativeBuffer.address());
        }
        catch (WindowsException ex2) {
            throw new IOException(str + ": " + ex2.errorString());
        }
        finally {
            nativeBuffer.release();
        }
    }
    
    static class User implements UserPrincipal
    {
        private final String sidString;
        private final int sidType;
        private final String accountName;
        
        User(String sidString, int sidType, String accountName) {
            this.sidString = sidString;
            this.sidType = sidType;
            this.accountName = accountName;
        }
        
        String sidString() {
            return this.sidString;
        }
        
        @Override
        public String getName() {
            return this.accountName;
        }
        
        @Override
        public String toString() {
            String str;
            switch (this.sidType) {
                case 1: {
                    str = "User";
                    break;
                }
                case 2: {
                    str = "Group";
                    break;
                }
                case 3: {
                    str = "Domain";
                    break;
                }
                case 4: {
                    str = "Alias";
                    break;
                }
                case 5: {
                    str = "Well-known group";
                    break;
                }
                case 6: {
                    str = "Deleted";
                    break;
                }
                case 7: {
                    str = "Invalid";
                    break;
                }
                case 9: {
                    str = "Computer";
                    break;
                }
                default: {
                    str = "Unknown";
                    break;
                }
            }
            return this.accountName + " (" + str + ")";
        }
        
        @Override
        public boolean equals(Object o) {
            return o == this || (o instanceof WindowsUserPrincipals.User && this.sidString.equals(((WindowsUserPrincipals.User)o).sidString));
        }
        
        @Override
        public int hashCode() {
            return this.sidString.hashCode();
        }
    }
    
    static class Group extends WindowsUserPrincipals.User implements GroupPrincipal
    {
        Group(String s, int n, String s2) {
            super(s, n, s2);
        }
    }
}
